#!/usr/bin/python
# -*- coding: utf-8 -*-

from PyQt5 import QtCore
from .model import MedvedModel
from .wmedeval import QWMEDeval
from . import utils, widgets


class MedvedController(QtCore.QObject):
    sigCancel = QtCore.pyqtSignal()

    def __init__(self):
        super().__init__()
        self.settings = QtCore.QSettings()
        self.threadUpdateChecker = None
        self.createModel()
        self.createWindows()
        self.deleteThreadUpdateChecker()
        self.connectSignals()

    def createUpdateChecker(self):
        self.checkUpdatesByUser = False
        self.threadUpdateChecker = QtCore.QThread()
        self.updateChecker = utils.UpdateChecker()
        self.updateChecker.moveToThread(self.threadUpdateChecker)
        # noinspection PyUnresolvedReferences
        self.threadUpdateChecker.started.connect(self.updateChecker.check)
        self.updateChecker.sigNewVersion.connect(self.showNewVersion)
        self.updateChecker.sigFinished.connect(self.deleteThreadUpdateChecker)

    def deleteThreadUpdateChecker(self):
        self.checkUpdatesByUser = False
        if self.threadUpdateChecker:
            self.threadUpdateChecker.quit()
            self.threadUpdateChecker.wait()
        self.threadUpdateChecker = None
        self.updateChecker = None

    def createModel(self):
        self.mthread = QtCore.QThread()
        self.model = MedvedModel()
        self.model.moveToThread(self.mthread)

    def loadSettings(self):
        s = self.settings
        self.wupdates.cb.setChecked(utils.pyqt2bool(s.value('Controller/doNotCheckUpdates', False)))
        self.wmedved.loadSettings(s)

    def saveSettings(self):
        s = self.settings
        s.setValue('Controller/doNotCheckUpdates', self.wupdates.cb.isChecked())
        self.wmedved.saveSettings(s)

    def createWindows(self):
        self.wmedved = QWMEDeval()
        self.wprogress = widgets.ProgressWindow(self.wmedved)
        self.wabout = widgets.WAbout(self.wmedved)
        self.wupdates = widgets.WUpdates(self.wmedved)

    def connectSignals(self):
        self.connectGUISignals()
        self.connectModelSignals()

    def connectGUISignals(self):
        self.wmedved.sigClose.connect(self.closeAll)
        self.wmedved.sigDir.connect(self.wprogress.show)
        self.wmedved.sigDir.connect(self.model.loadData)
        self.wmedved.sigData2Th.connect(self.model.getData2Th)
        self.wmedved.sigDataTime.connect(self.model.getDataTime)
        self.wmedved.sigFourier2Th.connect(self.model.getFourier2Th)
        self.wmedved.sigFourierTime.connect(self.model.getFourierTime)
        self.wmedved.sigUpdateFourierPlot.connect(self.model.getFourierImage)
        self.wmedved.sigUpdateDataPlot.connect(self.model.getDataImage)
        self.wmedved.sigFourierDomain.connect(self.model.setFourierDomain)
        self.wmedved.sigUnits.connect(self.model.setUnits)
        self.wmedved.sigWavelength.connect(self.model.setWavelength)
        self.wmedved.sigOpenFiles.connect(self.model.loadData)
        self.wmedved.sigClean.connect(self.model.cleanData)
        self.wmedved.sigClean.connect(self.clean)
        self.sigCancel.connect(self.model.cancelLoad)
        self.wprogress.sigCancel.connect(self.sigCancel.emit)
        self.wmedved.dataTree.sigItemMoved.connect(self.model.dataOrderChanged)
        self.wmedved.actionAbout.triggered.connect(self.wabout.exec)
        self.wmedved.actionCheckUpdates.triggered.connect(self.checkNewVersionByUser)
        self.wabout.buttonUpdate.clicked.connect(self.checkNewVersionByUser)

    def connectModelSignals(self):
        self.model.sigLoadingError.connect(self.wmedved.showModelError)
        self.model.sigLoadingError.connect(self.finishedLoad)
        self.model.sigFinishedLoading.connect(self.wmedved.finishedLoad)
        self.model.sigFinishedLoading.connect(self.finishedLoad)
        self.model.sigDataImage.connect(self.plotDataImage)
        self.model.sigFourierImage.connect(self.plotFourierImage)
        self.model.sigData2Th.connect(self.showData2Th)
        self.model.sigDataTime.connect(self.showDataTime)
        self.model.sigFourier2Th.connect(self.showFourier2Th)
        self.model.sigFourierTime.connect(self.showFourierTime)
        self.model.sigLoadProgress.connect(self.wprogress.setValue)
        self.model.sigMaxProgress.connect(self.wprogress.setMaximum)
        self.model.sigProgressName.connect(self.wprogress.setText)

    def finishedLoad(self):
        self.wprogress.hide()
        self.wmedved.dataSliders.timeSlider.valueChanged.emit(0)
        self.wmedved.fourierSliders.timeSlider.valueChanged.emit(0)

    def plotDataImage(self, data):
        self.wmedved.dataSliders.imageView.setImage(data)

    def plotFourierImage(self, data):
        self.wmedved.fourierSliders.imageView.setImage(data)

    def showData2Th(self, value, s, x, y):
        self.wmedved.dataSliders.plot.plot(x, y, pen='g', clear=True)
        self.wmedved.dataSliders.twothEdit.setText(s)
        self.wmedved.dataSliders.imageView.drawArea(value, self.wmedved.dataSliders.twothSlider.orientation())

    def showDataTime(self, value, x, y):
        self.wmedved.dataSliders.timeEdit.setText(str(value))
        self.wmedved.dataSliders.plot.plot(x, y, pen='g', clear=True)
        self.wmedved.dataSliders.imageView.drawArea(value, self.wmedved.dataSliders.timeSlider.orientation())

    def showFourier2Th(self, value, units, x, y):
        self.wmedved.fourierSliders.twothEdit.setText(str(value))
        self.wmedved.fourierSliders.plot.plot(x, y, clear=True, **self.getPlotParams(units))
        self.wmedved.fourierSliders.plot.plotItem.axes['left']['item'].setTicks(self.getPlotTicks(units))
        self.wmedved.fourierSliders.imageView.drawArea(value, self.wmedved.fourierSliders.twothSlider.orientation())

    def showFourierTime(self, value, units, x, y):
        self.wmedved.fourierSliders.timeEdit.setText(str(value))
        self.wmedved.fourierSliders.plot.plot(x, y, clear=True, **self.getPlotParams(units))
        self.wmedved.fourierSliders.plot.plotItem.axes['left']['item'].setTicks(self.getPlotTicks(units))
        self.wmedved.fourierSliders.imageView.drawArea(value, self.wmedved.fourierSliders.timeSlider.orientation())

    def getPlotTicks(self, units):
        if units == 'p':
            return [[(-180, '-π'), (-90, '-π/2'), (0, '0'), (90, 'π/2'), (180, 'π')]]

    def getPlotParams(self, units):
        if units == 'p':
            return {'pen': None, 'symbol': 'o'}
        else:
            return {'pen': 'g', 'symbol': None}

    def start(self):
        self.loadSettings()
        self.mthread.start()
        self.wmedved.start()
        self.checkNewVersion()

    def closeAll(self):
        self.saveSettings()
        self.sigCancel.emit()
        self.wmedved.hide()
        self.mthread.quit()
        self.deleteThreadUpdateChecker()
        self.mthread.wait()

    def clean(self):
        pass

    def checkNewVersion(self):
        if not self.wupdates.cb.isChecked():
            self.createUpdateChecker()
            self.threadUpdateChecker.start()

    def showNewVersion(self, text, error):
        if self.checkUpdatesByUser:
            self.wupdates.error(text) if error else self.wupdates.success(text)
        else:
            if not error and not self.wupdates.cb.isChecked() and text:
                self.wupdates.success(text)

    def checkNewVersionByUser(self):
        self.createUpdateChecker()
        self.checkUpdatesByUser = True
        self.threadUpdateChecker.start()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'medved/ui//ui_wabout.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WAbout(object):
    def setupUi(self, WAbout):
        WAbout.setObjectName("WAbout")
        WAbout.setWindowModality(QtCore.Qt.ApplicationModal)
        WAbout.resize(868, 762)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/medeval"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        WAbout.setWindowIcon(icon)
        WAbout.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.verticalLayout = QtWidgets.QVBoxLayout(WAbout)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.label = QtWidgets.QLabel(WAbout)
        self.label.setText("")
        self.label.setPixmap(QtGui.QPixmap(":/medeval"))
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.aboutLabel = QtWidgets.QLabel(WAbout)
        self.aboutLabel.setOpenExternalLinks(True)
        self.aboutLabel.setTextInteractionFlags(QtCore.Qt.TextBrowserInteraction)
        self.aboutLabel.setObjectName("aboutLabel")
        self.horizontalLayout_3.addWidget(self.aboutLabel)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem3)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.buttonUpdate = QtWidgets.QPushButton(WAbout)
        self.buttonUpdate.setObjectName("buttonUpdate")
        self.horizontalLayout.addWidget(self.buttonUpdate)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem4)
        self.aboutQtButton = QtWidgets.QPushButton(WAbout)
        self.aboutQtButton.setObjectName("aboutQtButton")
        self.horizontalLayout.addWidget(self.aboutQtButton)
        self.closeButton = QtWidgets.QPushButton(WAbout)
        self.closeButton.setObjectName("closeButton")
        self.horizontalLayout.addWidget(self.closeButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(WAbout)
        QtCore.QMetaObject.connectSlotsByName(WAbout)

    def retranslateUi(self, WAbout):
        _translate = QtCore.QCoreApplication.translate
        WAbout.setWindowTitle(_translate("WAbout", "About MEDVED"))
        self.aboutLabel.setText(_translate("WAbout", "<html><head/><body><p>MEDVED (Modulation-Enhanced Viewer and EDitor)</p><p>(c) 2016-2017, Vadim Dyadkin, SNBL@ESRF</p><p>This program is licensed under GPL v3</p><p>Mercurial repository: <a href=\"https://hg.3lp.cx/medved\"><span style=\" text-decoration: underline; color:#0057ae;\">https://hg.3lp.cx/medved</span></a></p><p>Version: @@</p><p>Mercurial hash: ##</p><p>When you use this software, please quote the following reference:</p><p><a href=\"http://dx.doi.org/10.1107/S2053273316008378\"><span style=\" text-decoration: underline; color:#0057ae;\">http://dx.doi.org/10.1107/S2053273316008378</span></a></p><p>More information at <a href=\"https://soft.snbl.eu\"><span style=\" text-decoration: underline; color:#0057ae;\">https://soft.snbl.eu</span></a></p></body></html>"))
        self.buttonUpdate.setText(_translate("WAbout", "Check updates"))
        self.aboutQtButton.setText(_translate("WAbout", "About Qt"))
        self.closeButton.setText(_translate("WAbout", "Close"))

from . import resources_rc



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import glob
import concurrent.futures
import numpy as np
from scipy import fftpack
from PyQt5 import QtCore


class MedvedModel(QtCore.QObject):
    sigProgressName = QtCore.pyqtSignal(str)
    sigFinishedLoading = QtCore.pyqtSignal(int, int, int, int, tuple)
    sigLoadingError = QtCore.pyqtSignal(str)
    sigLoadProgress = QtCore.pyqtSignal(int)
    sigMaxProgress = QtCore.pyqtSignal(int)
    sigDataImage = QtCore.pyqtSignal(np.ndarray)
    sigFourierImage = QtCore.pyqtSignal(np.ndarray)
    sigData2Th = QtCore.pyqtSignal(int, str, np.ndarray, np.ndarray)
    sigDataTime = QtCore.pyqtSignal(int, np.ndarray, np.ndarray)
    sigFourier2Th = QtCore.pyqtSignal(int, str, np.ndarray, np.ndarray)
    sigFourierTime = QtCore.pyqtSignal(int, str, np.ndarray, np.ndarray)

    def __init__(self):
        super().__init__()
        self.canceled = True
        self.wavelength = 0.7
        self.units = 't'
        self.fdomain = 'r'
        self.cleanData()
        self.connectSignals()

    def cleanData(self):
        self.data = None
        self.fdata = None
        self.files = []
        self.dataFiles = []
        self.fdata = None
        self.frange1 = None
        self.absolute = None
        self.phase = None
        self.datarange = None
        self.data_d = None
        self.data_dd = None
        self.lastData = lambda: None
        self.lastFourier = lambda: None

    def connectSignals(self):
        pass

    def loadData(self, files):
        if isinstance(files, str) and os.path.isdir(files):
            files = glob.glob(f'{files}/*')
        elif isinstance(files, list):
            pass
        else:
            return
        self.canceled = False
        self.sigProgressName.emit('Reading files...')
        self.sigMaxProgress.emit(len(files))
        self.readFiles(files)
        self.calc()
        if not self.canceled and not self.files:
            self.sigLoadingError.emit('There were no acceptable files')
            self.cleanData()
        self.canceled = True

    def calc(self):
        if not self.dataFiles or self.canceled:
            return
        self.data = np.empty(self.dataFiles[0].shape + (len(self.dataFiles),))
        for i, d in enumerate(self.dataFiles):
            try:
                self.data[:, :, i] = d
            except IndexError:
                self.sigLoadingError.emit('There was an error with loading of files')
                self.cleanData()
                return
        self.calc_d()
        self.calculateFourier()
        if self.data is not None and self.fdata is not None:
            self.sigFinishedLoading.emit(self.data.shape[2] - 1, self.fdata.shape[1] - 1,
                                         self.data.shape[0] - 1, self.fdata.shape[0] - 1,
                                         self.files)
            self.getDataImage()
            self.getFourierImage()

    def calc_d(self):
        if self.data is not None:
            self.datarange = np.arange(self.data.shape[2])
            self.data_d = self.wavelength / 2 / np.sin(np.radians(self.data[:, 0, :]) / 2)
            self.data_dd = self.data_d ** 2

    def calculateFourier(self):
        if self.data is not None:
            self.fdata = fftpack.fft(self.data[:, 1, :])
            self.frange1 = np.arange(self.fdata.shape[1])
            self.absolute = np.absolute(self.fdata)
            self.phase = np.angle(self.fdata, deg=True)

    def cancelLoad(self):
        self.canceled = True

    def readFiles(self, files):
        f, d = [], []
        with concurrent.futures.ThreadPoolExecutor(max_workers=os.cpu_count()) as executor:
            futures = {executor.submit(read_file, file): file for file in files}
            for i, future in enumerate(concurrent.futures.as_completed(futures)):
                QtCore.QCoreApplication.processEvents()
                if self.canceled:
                    for f in futures:
                        f.cancel()
                    self.cleanData()
                    return
                result = future.result()
                if result is not None:
                    d.append(result[0])
                    f.append(result[1])
                self.sigLoadProgress.emit(i)
        self.files, self.dataFiles = zip(*sorted(zip(f, d)))

    def getData2Th(self, value):
        if self.data is None:
            return
        x, y = self.datarange, self.data[value, 1, :]
        s = str(self.data[value, 0, 0])
        self.sigData2Th.emit(value, s, x, y)
        self.lastData = lambda: self.getData2Th(value)

    def getDataTime(self, value):
        if self.data is None:
            return
        x = self.getDataSlice(value)
        if x is None:
            return
        y = self.data[:, 1, value]
        self.lastData = lambda: self.getDataTime(value)
        self.sigDataTime.emit(value, x, y)

    def getDataSlice(self, value):
        if self.units == 't':
            return self.data[:, 0, value]
        elif self.units == 'd':
            return self.data_d[:, value]
        elif self.units == 'dd':
            return self.data_dd[:, value]

    def getFourierSlice(self, s1, s2):
        if self.fdomain == 'i':
            return self.fdata[s1, s2].imag
        elif self.fdomain == 'a':
            return self.absolute[s1, s2]
        elif self.fdomain == 'p':
            return self.phase[s1, s2]
        elif self.fdomain == 'r':
            return self.fdata[s1, s2].real

    def getFourier2Th(self, value):
        if self.fdata is None:
            return
        y = self.getFourierSlice(value, slice(None))
        if y is None:
            return
        x = self.frange1
        self.sigFourier2Th.emit(value, self.fdomain, x, y)
        self.lastFourier = lambda: self.getFourier2Th(value)

    def getFourierTime(self, value):
        if self.fdata is None:
            return
        x = self.getDataSlice(value)
        y = self.getFourierSlice(slice(None), value)
        if x is None or y is None:
            return
        self.sigFourierTime.emit(value, self.fdomain, x, y)
        self.lastFourier = lambda: self.getFourierTime(value)

    def getFourierImage(self):
        if self.fdata is not None:
            self.sigFourierImage.emit(self.getFourierSlice(slice(None), slice(None)))
            self.lastFourier()

    def getDataImage(self):
        if self.data is not None:
            self.sigDataImage.emit(self.data[:, 1, :])
            self.lastData()

    def dataOrderChanged(self, old, new):
        if old > new:
            s, c = slice(new, old + 1), 1
        elif old < new:
            s, c = slice(old, new + 1), -1
        else:
            return
        self.data[:, 1, s] = np.roll(self.data[:, 1, s], c, axis=1)
        self.calculateFourier()
        self.getDataImage()
        self.getFourierImage()

    def setWavelength(self, wavelength):
        try:
            self.wavelength = float(wavelength)
        except ValueError:
            return
        self.calc_d()
        self.lastData()
        self.lastFourier()

    def setFourierDomain(self, domain):
        self.fdomain = domain

    def setUnits(self, units):
        self.units = units


def read_file(datafile):
    if not os.path.isfile(datafile):
        return
    if datafile.endswith('.hkl'):
        data = read_hkl(datafile)
    else:
        try:
            data = np.loadtxt(datafile)
        except ValueError:
            return
    return data, os.path.basename(datafile)


def read_hkl(datafile):
    # noinspection PyUnresolvedReferences
    header = ff.FortranRecordReader('3I4,2F8.2')
    lst = []
    for line in open(datafile):
        try:
            line = header.readDir(line)
        except ValueError:
            break
        else:
            if None not in line:
                lst.append(line)
    array = np.array(lst)
    array = array[np.lexsort((array[:, 2], array[:, 1], array[:, 0]))]  # sorting by l k h
    b = np.empty_like(array)
    it = np.nditer(array, flags=['multi_index'])
    hkl = np.array([0., 0., 0., 0., 0.])
    j, k = 0, 1
    first = True
    while not it.finished:
        index = it.multi_index[1]
        hkl[index] = it[0]
        if index == 4:
            if first:
                first = False
                _hkl = hkl.copy()
            elif np.array_equal(hkl[:3], _hkl[:3]):
                k += 1
                _hkl[3:] += hkl[3:]
            else:
                _hkl[3:] /= k
                b[j, :] = _hkl
                _hkl = hkl.copy()
                k = 1
                j += 1
        it.iternext()
    array = np.c_[np.arange(j), b[:j, 3:], b[:j, :3]]
    return array



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'medved/ui//ui_wmedeval.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MEDeval(object):
    def setupUi(self, MEDeval):
        MEDeval.setObjectName("MEDeval")
        MEDeval.resize(900, 674)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/medeval"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MEDeval.setWindowIcon(icon)
        MEDeval.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.centralwidget = QtWidgets.QWidget(MEDeval)
        self.centralwidget.setObjectName("centralwidget")
        MEDeval.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MEDeval)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 900, 49))
        self.menubar.setObjectName("menubar")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MEDeval.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MEDeval)
        self.statusbar.setObjectName("statusbar")
        MEDeval.setStatusBar(self.statusbar)
        self.toolBar = QtWidgets.QToolBar(MEDeval)
        self.toolBar.setObjectName("toolBar")
        MEDeval.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.dock = QtWidgets.QDockWidget(MEDeval)
        self.dock.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.dock.setObjectName("dock")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.dock.setWidget(self.dockWidgetContents)
        MEDeval.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dock)
        self.actionOpenDir = QtWidgets.QAction(MEDeval)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/folder"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOpenDir.setIcon(icon1)
        self.actionOpenDir.setObjectName("actionOpenDir")
        self.actionReal = QtWidgets.QAction(MEDeval)
        self.actionReal.setCheckable(True)
        self.actionReal.setChecked(True)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/re"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionReal.setIcon(icon2)
        self.actionReal.setObjectName("actionReal")
        self.actionImaginary = QtWidgets.QAction(MEDeval)
        self.actionImaginary.setCheckable(True)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/im"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionImaginary.setIcon(icon3)
        self.actionImaginary.setObjectName("actionImaginary")
        self.actionAmplitude = QtWidgets.QAction(MEDeval)
        self.actionAmplitude.setCheckable(True)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/ampl"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAmplitude.setIcon(icon4)
        self.actionAmplitude.setObjectName("actionAmplitude")
        self.actionPhase = QtWidgets.QAction(MEDeval)
        self.actionPhase.setCheckable(True)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/phi"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionPhase.setIcon(icon5)
        self.actionPhase.setObjectName("actionPhase")
        self.action2Theta = QtWidgets.QAction(MEDeval)
        self.action2Theta.setCheckable(True)
        self.action2Theta.setChecked(True)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/theta"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action2Theta.setIcon(icon6)
        self.action2Theta.setObjectName("action2Theta")
        self.actionD = QtWidgets.QAction(MEDeval)
        self.actionD.setCheckable(True)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/d"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionD.setIcon(icon7)
        self.actionD.setObjectName("actionD")
        self.actionD2 = QtWidgets.QAction(MEDeval)
        self.actionD2.setCheckable(True)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/d2"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionD2.setIcon(icon8)
        self.actionD2.setObjectName("actionD2")
        self.actionAbout = QtWidgets.QAction(MEDeval)
        self.actionAbout.setObjectName("actionAbout")
        self.actionQuit = QtWidgets.QAction(MEDeval)
        self.actionQuit.setObjectName("actionQuit")
        self.actionOpenFiles = QtWidgets.QAction(MEDeval)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/file"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOpenFiles.setIcon(icon9)
        self.actionOpenFiles.setObjectName("actionOpenFiles")
        self.actionListDock = QtWidgets.QAction(MEDeval)
        self.actionListDock.setCheckable(True)
        self.actionListDock.setChecked(False)
        self.actionListDock.setObjectName("actionListDock")
        self.actionClean = QtWidgets.QAction(MEDeval)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/clean"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionClean.setIcon(icon10)
        self.actionClean.setObjectName("actionClean")
        self.actionCheckUpdates = QtWidgets.QAction(MEDeval)
        self.actionCheckUpdates.setObjectName("actionCheckUpdates")
        self.menu_File.addAction(self.actionOpenFiles)
        self.menu_File.addAction(self.actionOpenDir)
        self.menu_File.addAction(self.actionClean)
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.actionQuit)
        self.menuView.addAction(self.actionListDock)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionReal)
        self.menuView.addAction(self.actionImaginary)
        self.menuView.addAction(self.actionAmplitude)
        self.menuView.addAction(self.actionPhase)
        self.menuView.addSeparator()
        self.menuView.addAction(self.action2Theta)
        self.menuView.addAction(self.actionD)
        self.menuView.addAction(self.actionD2)
        self.menuHelp.addAction(self.actionCheckUpdates)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())
        self.toolBar.addAction(self.actionOpenFiles)
        self.toolBar.addAction(self.actionOpenDir)
        self.toolBar.addAction(self.actionClean)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.actionReal)
        self.toolBar.addAction(self.actionImaginary)
        self.toolBar.addAction(self.actionAmplitude)
        self.toolBar.addAction(self.actionPhase)
        self.toolBar.addSeparator()
        self.toolBar.addAction(self.action2Theta)
        self.toolBar.addAction(self.actionD)
        self.toolBar.addAction(self.actionD2)

        self.retranslateUi(MEDeval)
        QtCore.QMetaObject.connectSlotsByName(MEDeval)

    def retranslateUi(self, MEDeval):
        _translate = QtCore.QCoreApplication.translate
        MEDeval.setWindowTitle(_translate("MEDeval", "MEDved"))
        self.menu_File.setTitle(_translate("MEDeval", "&File"))
        self.menuView.setTitle(_translate("MEDeval", "&View"))
        self.menuHelp.setTitle(_translate("MEDeval", "&Help"))
        self.toolBar.setWindowTitle(_translate("MEDeval", "toolBar"))
        self.actionOpenDir.setText(_translate("MEDeval", "&Open directory..."))
        self.actionOpenDir.setShortcut(_translate("MEDeval", "Ctrl+F"))
        self.actionReal.setText(_translate("MEDeval", "&Real part"))
        self.actionReal.setToolTip(_translate("MEDeval", "Show real part"))
        self.actionReal.setShortcut(_translate("MEDeval", "Ctrl+R"))
        self.actionImaginary.setText(_translate("MEDeval", "&Imaginary part"))
        self.actionImaginary.setToolTip(_translate("MEDeval", "Show imaginary part"))
        self.actionImaginary.setShortcut(_translate("MEDeval", "Ctrl+I"))
        self.actionAmplitude.setText(_translate("MEDeval", "&Amplitute"))
        self.actionAmplitude.setToolTip(_translate("MEDeval", "Show amplitude"))
        self.actionAmplitude.setShortcut(_translate("MEDeval", "Ctrl+A"))
        self.actionPhase.setText(_translate("MEDeval", "&Phase"))
        self.actionPhase.setToolTip(_translate("MEDeval", "Show phase"))
        self.actionPhase.setShortcut(_translate("MEDeval", "Ctrl+P"))
        self.action2Theta.setText(_translate("MEDeval", "&2Theta"))
        self.action2Theta.setShortcut(_translate("MEDeval", "Ctrl+T"))
        self.actionD.setText(_translate("MEDeval", "&d"))
        self.actionD.setShortcut(_translate("MEDeval", "Ctrl+D"))
        self.actionD2.setText(_translate("MEDeval", "d2"))
        self.actionD2.setShortcut(_translate("MEDeval", "Ctrl+2"))
        self.actionAbout.setText(_translate("MEDeval", "&About..."))
        self.actionQuit.setText(_translate("MEDeval", "&Quit"))
        self.actionQuit.setShortcut(_translate("MEDeval", "Ctrl+Q"))
        self.actionOpenFiles.setText(_translate("MEDeval", "Open &files..."))
        self.actionOpenFiles.setShortcut(_translate("MEDeval", "Ctrl+O"))
        self.actionListDock.setText(_translate("MEDeval", "&List dock"))
        self.actionListDock.setShortcut(_translate("MEDeval", "Ctrl+L"))
        self.actionClean.setText(_translate("MEDeval", "&Clean"))
        self.actionClean.setShortcut(_translate("MEDeval", "Ctrl+Z"))
        self.actionCheckUpdates.setText(_translate("MEDeval", "Check updates..."))

from . import resources_rc



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
from PyQt5 import QtWidgets
from .controller import MedvedController


def main():
    app = QtWidgets.QApplication(sys.argv)
    app.setOrganizationName('SNBL')
    app.setOrganizationDomain('snbl.eu')
    app.setApplicationName('medved')
    c = MedvedController()
    c.start()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
from PyQt5 import QtCore, QtGui, QtWidgets
from . import widgets, pgwidgets
from .ui.ui_wmedeval import Ui_MEDeval


FOURIER_COMPONENTS = 'Real', 'Imaginary', 'Amplitude', 'Phase',


class QWMEDeval(QtWidgets.QMainWindow, Ui_MEDeval):
    sigClose = QtCore.pyqtSignal()
    sigDir = QtCore.pyqtSignal(str)
    sigCancelLoading = QtCore.pyqtSignal()
    sigUpdateFourierPlot = QtCore.pyqtSignal()
    sigUpdateDataPlot = QtCore.pyqtSignal()
    sigOpenFiles = QtCore.pyqtSignal(list)
    sigClean = QtCore.pyqtSignal()
    sigWavelength = QtCore.pyqtSignal(str)
    sigData2Th = QtCore.pyqtSignal(int)
    sigDataTime = QtCore.pyqtSignal(int)
    sigFourier2Th = QtCore.pyqtSignal(int)
    sigFourierTime = QtCore.pyqtSignal(int)
    sigUnits = QtCore.pyqtSignal(str)
    sigFourierDomain = QtCore.pyqtSignal(str)

    def __init__(self):
        super().__init__()
        self.setUI()
        self.connectSignals()

    def start(self):
        self.tabSwitched(0)
        self.uncheckAllXActionsBut(self.action2Theta)
        self.uncheckAllYActionsBut(self.actionReal)
        self.show()

    # noinspection PyUnresolvedReferences
    def connectSignals(self):
        self.dataSliders.timeSlider.valueChanged.connect(self.showCurrentDataTreeItem)
        self.fourierSliders.timeSlider.valueChanged.connect(self.showCurrentFourierTreeItem)
        self.tabWidget.currentChanged[int].connect(self.tabSwitched)
        self.wlEdit.textChanged[str].connect(self.sigWavelength.emit)
        self.dataSliders.twothSlider.valueChanged[int].connect(self.sigData2Th.emit)
        self.dataSliders.timeSlider.valueChanged[int].connect(self.sigDataTime.emit)
        self.fourierSliders.twothSlider.valueChanged[int].connect(self.sigFourier2Th.emit)
        self.fourierSliders.timeSlider.valueChanged[int].connect(self.sigFourierTime.emit)
        self.dataTree.itemClicked[QtWidgets.QTreeWidgetItem, int].connect(self.dataTreeItemClicked)
        self.fourierTree.itemClicked[QtWidgets.QTreeWidgetItem, int].connect(self.fourierTreeItemClicked)
        self.dataTree.sigItemMoved.connect(self.dataItemMoved)

    def dataItemMoved(self, old, new):
        # self.items.insert(new, self.items.pop(old))
        pass

    def dts(self):
        if self.action2Theta.isChecked():
            return 't'
        elif self.actionD.isChecked():
            return 'd'
        elif self.actionD2.isChecked():
            return 'dd'
        else:
            self.uncheckAllXActionsBut(self.action2Theta)
            return 't'

    def cfd(self):
        if self.actionAmplitude.isChecked():
            return 'a'
        elif self.actionImaginary.isChecked():
            return 'i'
        elif self.actionReal.isChecked():
            return 'r'
        elif self.actionPhase.isChecked():
            return 'p'
        else:
            self.uncheckAllYActionsBut(self.actionReal)
            return 'r'

    def tabSwitched(self, page):
        if page == 0:
            self.widgetFourier.hide()
            self.widgetData.show()
        elif page == 1:
            self.widgetData.hide()
            self.widgetFourier.show()

    def setUI(self):
        self.setupUi(self)
        self.tabWidget = QtWidgets.QTabWidget()
        self.dock.setWidget(self.tabWidget)
        self.setData()
        self.setFourier()
        self.widgetMain = QtWidgets.QWidget()
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.widgetData)
        layout.addWidget(self.widgetFourier)
        self.widgetMain.setLayout(layout)
        self.setCentralWidget(self.widgetMain)
        self.toolBar.addSeparator()
        self.wlEdit = QtWidgets.QLineEdit()
        self.wlEdit.setValidator(QtGui.QDoubleValidator())
        self.wlEdit.setMaximumWidth(200)
        self.toolBar.insertWidget(self.actionD, self.wlEdit)

    def updateStatusBar(self, x, y):
        self.statusbar.showMessage(f'x = {x:.6f}, y = {y:.6f}')

    def setData(self):
        self.widgetData = QtWidgets.QWidget()
        self.splitterData = QtWidgets.QSplitter(QtCore.Qt.Vertical, self.widgetData)
        self.dataPlot = pgwidgets.PlotWidget()
        self.dataPlot.sigWatchMouse.connect(self.updateStatusBar)
        self.dataSliders = widgets.WSliders(self.dataPlot, self)
        self.splitterData.addWidget(self.dataSliders)
        self.splitterData.addWidget(self.dataPlot)
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.splitterData)
        self.widgetData.setLayout(layout)
        self.dataTree = widgets.TreeWidget()
        self.dataTree.setHeaderLabel('Patterns')
        self.tabWidget.addTab(self.dataTree, 'Data')

    def setFourier(self):
        self.widgetFourier = QtWidgets.QWidget()
        self.splitterFourier = QtWidgets.QSplitter(QtCore.Qt.Vertical, self.widgetFourier)
        self.fourierPlot = pgwidgets.PlotWidget()
        self.fourierPlot.sigWatchMouse.connect(self.updateStatusBar)
        self.fourierSliders = widgets.WSliders(self.fourierPlot, self)
        self.splitterFourier.addWidget(self.fourierSliders)
        self.splitterFourier.addWidget(self.fourierPlot)
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.splitterFourier)
        self.widgetFourier.setLayout(layout)
        self.fourierTree = QtWidgets.QTreeWidget()
        self.fourierTree.setHeaderLabel('Harmonics')
        self.tabWidget.addTab(self.fourierTree, 'Fourier')
        self.widgetFourier.hide()

    def saveSettings(self, s):
        s.setValue('WMedved/Geometry', self.saveGeometry())
        s.setValue('WMedved/State', self.saveState())
        s.setValue('WMedved/lastdir', self.lastdir)
        s.setValue('WMedved/splitterFourierGeometry', self.splitterFourier.saveGeometry())
        s.setValue('WMedved/splitterFourierState', self.splitterFourier.saveState())
        s.setValue('WMedved/splitterDataGeometry', self.splitterData.saveGeometry())
        s.setValue('WMedved/splitterDataState', self.splitterData.saveState())
        s.setValue('WMedved/wavelength', self.wlEdit.text())

    def loadSettings(self, s):
        self.restoreGeometry(s.value('WMedved/Geometry', QtCore.QByteArray()))
        self.restoreState(s.value('WMedved/State', QtCore.QByteArray()))
        self.lastdir = s.value('WMedved/lastdir', '')
        self.splitterFourier.restoreGeometry(s.value('WMedved/splitterFourierGeometry', QtCore.QByteArray()))
        self.splitterFourier.restoreState(s.value('WMedved/splitterFourierState', QtCore.QByteArray()))
        self.splitterData.restoreGeometry(s.value('WMedved/splitterDataGeometry', QtCore.QByteArray()))
        self.splitterData.restoreState(s.value('WMedved/splitterDataState', QtCore.QByteArray()))
        self.wlEdit.setText(s.value('WMedved/wavelength', '0.7'))

    def closeEvent(self, event):
        self.sigClose.emit()
        super().closeEvent(event)

    def finishedLoad(self, t, ft, d, fd, files):
        self.dataSliders.twothSlider.setMaximum(d)
        self.dataSliders.timeSlider.setMaximum(t)
        self.fourierSliders.twothSlider.setMaximum(fd)
        self.fourierSliders.timeSlider.setMaximum(ft)
        self.dataTree.takeTopLevelItem(0)
        mainItem = QtWidgets.QTreeWidgetItem()
        # mainItem.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDropEnabled)
        mainItem.setText(0, 'Data files')
        self.dataTree.addTopLevelItem(mainItem)
        self.items = []
        for f in files:
            item = QtWidgets.QTreeWidgetItem()
            # item.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsDragEnabled)
            item.setText(0, f)
            mainItem.addChild(item)
            self.items.append(item)
        for _ in FOURIER_COMPONENTS:
            self.fourierTree.takeTopLevelItem(0)
        self.fitems = {}
        for name in FOURIER_COMPONENTS:
            mainItem = QtWidgets.QTreeWidgetItem(self.fourierTree)
            mainItem.setText(0, name)
            self.fitems[name] = []
            for i in range(ft + 1):
                item = QtWidgets.QTreeWidgetItem(mainItem)
                item.setText(0, f'Harmonic {i:d}')
                self.fitems[name].append(item)
            self.fourierTree.addTopLevelItem(mainItem)

    @QtCore.pyqtSlot()
    def on_actionOpenDir_triggered(self):
        dire = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select directory', directory=self.lastdir)
        if dire:
            self.lastdir = dire
            self.sigDir.emit(dire)

    def showCurrentDataTreeItem(self, value):
        if self.items:
            self.dataTree.setCurrentItem(self.items[value])

    def showCurrentFourierTreeItem(self, value):
        if self.fitems:
            for component in FOURIER_COMPONENTS:
                if self.__dict__[f'action{component}'].isChecked():
                    self.fourierTree.setCurrentItem(self.fitems[component][value])

    def dataTreeItemClicked(self, item):
        self.dataSliders.timeSlider.setSliderPosition(self.items.index(item))

    def fourierTreeItemClicked(self, item):
        text = item.text(0)
        if text not in FOURIER_COMPONENTS:
            text = item.parent().text(0)
            self.fourierSliders.timeSlider.setSliderPosition(self.fitems[text].index(item))
        self.uncheckAllYActionsBut(self.__dict__[f'action{text}'])

    @QtCore.pyqtSlot()
    def on_actionReal_triggered(self):
        self.uncheckAllYActionsBut(self.actionReal)

    @QtCore.pyqtSlot()
    def on_actionImaginary_triggered(self):
        self.uncheckAllYActionsBut(self.actionImaginary)

    @QtCore.pyqtSlot()
    def on_actionPhase_triggered(self):
        self.uncheckAllYActionsBut(self.actionPhase)

    @QtCore.pyqtSlot()
    def on_actionAmplitude_triggered(self):
        self.uncheckAllYActionsBut(self.actionAmplitude)

    @QtCore.pyqtSlot()
    def on_action2Theta_triggered(self):
        self.uncheckAllXActionsBut(self.action2Theta)

    @QtCore.pyqtSlot()
    def on_actionD_triggered(self):
        self.uncheckAllXActionsBut(self.actionD)

    @QtCore.pyqtSlot()
    def on_actionD2_triggered(self):
        self.uncheckAllXActionsBut(self.actionD2)

    def uncheckAllYActionsBut(self, action):
        actions = self.actionReal, self.actionPhase, self.actionAmplitude, self.actionImaginary
        self.uncheckAllActionsBut(action, actions)

    def uncheckAllXActionsBut(self, action):
        actions = self.action2Theta, self.actionD, self.actionD2
        self.uncheckAllActionsBut(action, actions)

    def uncheckAllActionsBut(self, action, actions):
        for act in actions:
            if action != act:
                act.setChecked(False)
        action.setChecked(True)
        self.sigUnits.emit(self.dts())
        self.sigFourierDomain.emit(self.cfd())
        self.sigUpdateFourierPlot.emit()
        self.sigUpdateDataPlot.emit()

    def showModelError(self, msg):
        QtWidgets.QMessageBox.critical(self, 'Error', msg)

    @QtCore.pyqtSlot()
    def on_actionQuit_triggered(self):
        self.close()

    @QtCore.pyqtSlot(bool)
    def on_dock_visibilityChanged(self, visible):
        self.actionListDock.setChecked(visible)

    @QtCore.pyqtSlot(bool)
    def on_actionListDock_toggled(self, checked):
        self.dock.setVisible(checked)

    @QtCore.pyqtSlot()
    def on_actionOpenFiles_triggered(self):
        files = QtWidgets.QFileDialog.getOpenFileNames(self, 'Open data files', self.lastdir)[0]
        if files:
            self.sigOpenFiles.emit(files)
            self.lastdir = os.path.dirname(files[0])

    @QtCore.pyqtSlot()
    def on_actionClean_triggered(self):
        self.sigClean.emit()
        self.fourierTree.clear()
        self.dataTree.clear()
        self.dataPlot.plotItem.clear()
        self.fourierPlot.plotItem.clear()
        self.fourierSliders.imageView.clear()
        self.dataSliders.imageView.clear()
        self.items.clear()
        self.fitems.clear()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import subprocess
import requests
from PyQt5 import QtCore

try:
    from . import frozen
except ImportError:
    frozen = False

UPDATE_URL = 'https://soft.snbl.eu/_sources/medved.rst.txt'
VERSION_PATTERN = '.. MEDVED Versions'


def pyqt2bool(entry):
    return not (entry == 'false' or not entry)


def get_hg_hash():
    if not get_hg_hash.hash:
        if hasattr(sys, 'frozen') or frozen:
            get_hg_hash.hash = frozen.hg_hash
        else:
            path = os.path.dirname(os.path.dirname(__file__))
            try:
                pipe = subprocess.Popen(['hg', 'id', '-i', '-R', path], stdout=subprocess.PIPE)
                get_hg_hash.hash = pipe.stdout.read().decode()
            except OSError:
                get_hg_hash.hash = 'unknown'
        get_hg_hash.hash = get_hg_hash.hash.strip()
    return get_hg_hash.hash


get_hg_hash.hash = ''


def get_version():
    if not get_version.version:
        if hasattr(sys, 'frozen') or frozen:
            get_version.version = frozen.version
        else:
            get_version.version = '0.0.0'
    return get_version.version


get_version.version = ''


class UpdateChecker(QtCore.QObject):
    sigNewVersion = QtCore.pyqtSignal(str, bool)
    sigFinished = QtCore.pyqtSignal()

    def check(self):
        try:
            r = requests.get(UPDATE_URL)
        except requests.RequestException as err:
            self.sigNewVersion.emit(str(err), True)
        else:
            self.parse(r.text)
        self.sigFinished.emit()

    def parse(self, text):
        version = get_version()
        strings = []
        put = False
        boldline = False
        first = True
        parsing = False
        for line in text.split('\n'):
            if not line:
                continue
            if line == VERSION_PATTERN:
                parsing = True
                continue
            if parsing:
                if line.startswith('..'):
                    continue
                items = line.split()
                try:
                    [int(i) for i in items[1].split('.')]
                except (IndexError, ValueError):
                    pass
                else:
                    put = items[1] > version
                    boldline = True
                if put:
                    line = line.strip()
                    line = line[line.index(" ") + 1:]
                    if boldline:
                        boldline = False
                        if not first:
                            strings.append('</ul>')
                        strings.append(f'<span style="font-weight: bold; color: red">{line}</span>'
                                       f'<ul style="list-style-type:disc">')
                    else:
                        strings.append(f'<li>{line}</li>')
                    first = False
        if strings:
            strings.append('</ul>')
        self.sigNewVersion.emit('\n'.join(strings), False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import subprocess
from setuptools import setup

version = '2017.9.12.1'
frozen_name = 'medved/frozen.py'

try:
    # noinspection PyUnresolvedReferences
    import compile
    compile.main()
except ImportError:
    pass

we_run_setup = False
if not os.path.exists(frozen_name):
    we_run_setup = True
    hash_ = subprocess.Popen(['hg', 'id', '-i'], stdout=subprocess.PIPE).stdout.read().decode().strip()
    print(f'Medved mercurial hash is {hash_}')
    frozen = open(frozen_name, 'w')
    frozen.write(f'# -*- coding: utf-8 -*-\nhg_hash = "{hash_}"\nversion = "{version}"\n')
    frozen.close()


setup(
    name='medved',
    version=version,
    description='Modulation Enhanced Diffraction Viewer and EDitor',
    author='Vadim Dyadkin',
    author_email='diadkin@esrf.fr',
    url='https://soft.snbl.eu/medved.html',
    license='GPLv3',
    long_description='Do not forget to cite: https://doi.org/10.1107/S2053273316008378',
    install_requires=[
        'numpy>=1.9',
        'scipy>=0.10.0',
        'fortranformat>=0.2.5',
        'pyqtgraph>=0.10.0',
        'requests',
    ],
    entry_points={
        'console_scripts': [
            'medved=medved.__init__:main',
        ],
    },
    packages=[
        'medved',
        'medved.ui',
    ],
)

if we_run_setup:
    os.remove(frozen_name)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'medved/ui//ui_sliders.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_QSlidersWidget(object):
    def setupUi(self, QSlidersWidget):
        QSlidersWidget.setObjectName("QSlidersWidget")
        QSlidersWidget.resize(653, 388)
        self.gridLayout = QtWidgets.QGridLayout(QSlidersWidget)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSizeConstraint(QtWidgets.QLayout.SetMinimumSize)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.twothEdit = QtWidgets.QLineEdit(QSlidersWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.twothEdit.sizePolicy().hasHeightForWidth())
        self.twothEdit.setSizePolicy(sizePolicy)
        self.twothEdit.setMaximumSize(QtCore.QSize(75, 75))
        self.twothEdit.setMaxLength(10)
        self.twothEdit.setObjectName("twothEdit")
        self.horizontalLayout.addWidget(self.twothEdit)
        self.twothSlider = QtWidgets.QSlider(QSlidersWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.twothSlider.sizePolicy().hasHeightForWidth())
        self.twothSlider.setSizePolicy(sizePolicy)
        self.twothSlider.setOrientation(QtCore.Qt.Horizontal)
        self.twothSlider.setObjectName("twothSlider")
        self.horizontalLayout.addWidget(self.twothSlider)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 1, 1, 1)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout.setObjectName("verticalLayout")
        self.timeEdit = QtWidgets.QLineEdit(QSlidersWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeEdit.sizePolicy().hasHeightForWidth())
        self.timeEdit.setSizePolicy(sizePolicy)
        self.timeEdit.setMaximumSize(QtCore.QSize(75, 75))
        self.timeEdit.setMaxLength(10)
        self.timeEdit.setObjectName("timeEdit")
        self.verticalLayout.addWidget(self.timeEdit)
        self.timeSlider = QtWidgets.QSlider(QSlidersWidget)
        self.timeSlider.setOrientation(QtCore.Qt.Vertical)
        self.timeSlider.setInvertedAppearance(True)
        self.timeSlider.setInvertedControls(False)
        self.timeSlider.setTickPosition(QtWidgets.QSlider.NoTicks)
        self.timeSlider.setObjectName("timeSlider")
        self.verticalLayout.addWidget(self.timeSlider)
        self.gridLayout.addLayout(self.verticalLayout, 1, 0, 1, 1)
        self.imageView = ImageView(QSlidersWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.imageView.sizePolicy().hasHeightForWidth())
        self.imageView.setSizePolicy(sizePolicy)
        self.imageView.setObjectName("imageView")
        self.gridLayout.addWidget(self.imageView, 1, 1, 1, 1)

        self.retranslateUi(QSlidersWidget)
        QtCore.QMetaObject.connectSlotsByName(QSlidersWidget)

    def retranslateUi(self, QSlidersWidget):
        _translate = QtCore.QCoreApplication.translate
        QSlidersWidget.setWindowTitle(_translate("QSlidersWidget", "Form"))

from ..pgwidgets import ImageView



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PyQt5 import QtCore, QtWidgets, QtGui
from . import utils
from .ui.ui_wabout import Ui_WAbout
from .ui.ui_sliders import Ui_QSlidersWidget


class WSliders(QtWidgets.QWidget, Ui_QSlidersWidget):
    def __init__(self, plot, parent):
        super().__init__(parent)
        self.plot = plot
        self.setupUi(self)


class ProgressWindow(QtWidgets.QDialog):
    sigCancel = QtCore.pyqtSignal()

    def __init__(self, parent):
        super().__init__(parent)
        self.setupUI()
        self.connectSignals()

    # noinspection PyUnresolvedReferences
    def connectSignals(self):
        self.bb.rejected.connect(self.cancel)

    def setupUI(self):
        self.label = QtWidgets.QLabel()
        self.bb = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Cancel)
        self.pb = QtWidgets.QProgressBar()
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.label)
        layout.addWidget(self.pb)
        layout.addWidget(self.bb)
        self.setLayout(layout)
        self.setWindowModality(QtCore.Qt.WindowModal)

    def setValue(self, value):
        self.pb.setValue(value)
        if self.pb.maximum() == value:
            self.hide()

    def setMaximum(self, value):
        self.pb.setMaximum(value)

    def setText(self, text):
        self.label.setText(text)

    def showEvent(self, event):
        self.setValue(0)
        super().showEvent(event)

    def cancel(self):
        self.hide()
        self.sigCancel.emit()


class TreeWidget(QtWidgets.QTreeWidget):
    sigItemMoved = QtCore.pyqtSignal(int, int)

    def __init__(self):
        super().__init__()
        # self.setDragAndDrop()

    def setDragAndDrop(self):
        self.oldIndex = None
        self.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.setDragEnabled(True)
        self.viewport().setAcceptDrops(True)
        self.setDropIndicatorShown(True)
        self.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)

    def dropEvent(self, event: QtGui.QDropEvent):
        droppedIndex = self.indexAt(event.pos())
        if not droppedIndex.isValid() or not self.oldIndex:
            return
        super().dropEvent(event)
        dp = self.dropIndicatorPosition()
        if dp == QtWidgets.QAbstractItemView.BelowItem and self.oldIndex > droppedIndex:
            droppedIndex = droppedIndex.sibling(droppedIndex.row() + 1, droppedIndex.column())
        elif dp == QtWidgets.QAbstractItemView.AboveItem and self.oldIndex < droppedIndex:
            droppedIndex = droppedIndex.sibling(droppedIndex.row() - 1, droppedIndex.column())
        selectionModel = self.selectionModel()
        selectionModel.clearSelection()
        selectionModel.select(droppedIndex, QtCore.QItemSelectionModel.Select)
        self.sigItemMoved.emit(self.oldIndex.row(), droppedIndex.row())
        self.oldIndex = None

    def dragEnterEvent(self, event: QtGui.QDragEnterEvent):
        oldIndex = self.indexAt(event.pos())
        if oldIndex.isValid():
            super().dragEnterEvent(event)
            self.oldIndex = oldIndex


class WAbout(QtWidgets.QDialog, Ui_WAbout):
    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)
        lt = self.aboutLabel.text()
        lt = lt.replace('##', utils.get_hg_hash()).replace('@@', utils.get_version())
        self.aboutLabel.setText(lt)
        QtCore.QTimer.singleShot(0, lambda: self.resize(0, 0))

    @QtCore.pyqtSlot()
    def on_closeButton_clicked(self):
        self.close()

    @QtCore.pyqtSlot()
    def on_aboutQtButton_clicked(self):
        QtWidgets.QMessageBox.aboutQt(self)


class WUpdates(QtWidgets.QMessageBox):
    def __init__(self, parent):
        super().__init__(parent)
        self.setupUI()

    def setupUI(self):
        self.setWindowTitle('Medved updates')
        self.setStandardButtons(self.Ok)
        self.cb = QtWidgets.QCheckBox('Do not check updates automatically')
        self.setCheckBox(self.cb)

    def error(self, text):
        self.setIcon(self.Critical)
        self.setText('There has been an error during update checking')
        self.setInformativeText(text)
        self.exec()

    def success(self, text):
        self.setIcon(self.Information)
        if text:
            self.setText('A new MEDved version is available at <a href=https://soft.snbl.eu/medved.html#download>'
                         'soft.snbl.eu</a>')
            self.setInformativeText(text)
        else:
            self.setText('There are no new updates of MEDved')
        self.exec()



#!/usr/bin/python
# -*- coding: utf-8 -*-

from PyQt5 import QtCore, QtGui, QtWidgets
import pyqtgraph as pg


class ImageView(pg.ImageView):
    def __init__(self, *args, **kwargs):
        super(ImageView, self).__init__(*args, **kwargs)
        self.ui.roiBtn.hide()
        self.ui.menuBtn.hide()
        color = QtGui.QColor(255, 0, 0, 150)
        self.varea = QtWidgets.QGraphicsRectItem(0, 0, 1, 1)
        self.varea.setPen(QtGui.QPen(color))
        self.varea.setBrush(QtGui.QBrush(color))
        self.harea = QtWidgets.QGraphicsRectItem(0, 0, 1, 1)
        self.harea.setPen(QtGui.QPen(color))
        self.harea.setBrush(QtGui.QBrush(color))
        self.varea.setVisible(False)
        self.harea.setVisible(False)
        self.view.addItem(self.varea)
        self.view.addItem(self.harea)

    def setImage(self, *args, **kwargs):
        pg.ImageView.setImage(self, *args, **kwargs)
        levelMax = self.quickMinMax(self.image)[1]
        self.setLevels(-levelMax / 20, levelMax / 4)
        self.rescaleImage()

    def rescaleImage(self):
        if self.image is None:
            return
        viewRect = self.view.rect()
        self.imageItem.setRect(viewRect)
        self.view.autoRange()
        width, height = viewRect.width(), viewRect.height()
        self.hstep = height / self.imageItem.height()
        self.vstep = width / self.imageItem.width()
        self.harea.setRect(0, self.hstep, width, self.hstep + 1)
        self.varea.setRect(self.vstep, 0, self.vstep + 1, height)

    def drawArea(self, n, orientation):
        if self.image is None:
            return
        if orientation == QtCore.Qt.Vertical:
            self.varea.setVisible(False)
            self.harea.setVisible(True)
            self.harea.setY(self.hstep * (n - 1))
        elif orientation == QtCore.Qt.Horizontal:
            self.varea.setVisible(True)
            self.harea.setVisible(False)
            self.varea.setX(self.vstep * (n - 1))


class PlotWidget(pg.PlotWidget):
    sigWatchMouse = QtCore.pyqtSignal(float, float)

    def __init__(self, parent=None, background='default', **kargs):
        super().__init__(parent, background, **kargs)
        self.proxy = pg.SignalProxy(self.plotItem.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)

    def mouseMoved(self, evt):
        pos = evt[0]
        if self.sceneBoundingRect().contains(pos):
            mousePoint = self.plotItem.vb.mapSceneToView(pos)
            self.sigWatchMouse.emit(mousePoint.x(), mousePoint.y())
